# -*- encoding: utf-8 -*-
require 'spec_helper'
#コマの画像背景

describe GroundPicture do
  before do
    FactoryGirl.create :admin
    @user = FactoryGirl.create( :user_yas)
    @author = @user.author
    @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
    @other_user = FactoryGirl.create( :user_yas)
    @other_author = @other_user.author
    @other_artist = FactoryGirl.create :artist_yas, :author_id => @other_author.id
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
    @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
    @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id, :picture_id => @p.id
    @panel = FactoryGirl.create :panel, :author_id => @author.id
  end
  
  describe '検証に於いて' do
    before do
      @gp = FactoryGirl.build :ground_picture, :panel_id => @panel.id, :picture_id => @p.id
    end
    
    context 'オーソドックスなデータのとき' do
      it '下限データが通る' do
        @gp.z = 1
        @gp.should be_valid
      end
      it '上限データが通る' do
        @gp.z = 99999
        @gp.should be_valid
      end
    end
    
    
    context 'panel_idを検証するとき' do
      #ネストの保存はnilを許可しなければならないので数値チェックだけ
      it '数値でなければ失敗する' do
        @gp.panel_id = 'a'
        @gp.should_not be_valid
      end
    end
    context 'picture_idを検証するとき' do
      it 'nullなら失敗する' do
        @gp.picture_id = nil
        @gp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @gp.picture_id = 'a'
        @gp.should_not be_valid
      end
      it '存在する実素材でなければ失敗する' do
        @gp.picture_id = 0
        @gp.should_not be_valid
      end
    end
    context 'zを検証するとき' do
      it 'nullなら失敗する' do
        @gp.z = nil
        @gp.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @gp.z = 'a'
        @gp.should_not be_valid
      end
      it '負なら失敗する' do
        @gp.z = -1
        @gp.should_not be_valid
      end
      it '負なら失敗する' do
        @gp.z = 0
        @gp.should_not be_valid
      end
    end
  end
  
  describe 'デフォルト値補充に於いて' do
    it 'defined' do
      @gp = FactoryGirl.build :ground_picture, :panel_id => @panel.id, :picture_id => @p.id
      @gp.supply_default
    end
  end
  
  describe '上書き補充に於いて' do
    it 'defined' do
      @gp = FactoryGirl.build :ground_picture, :panel_id => @panel.id, :picture_id => @p.id
      @gp.overwrite
    end
  end
  
  describe '一覧取得に於いて' do
    before do
      @gp = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :picture_id => @p.id
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        GroundPicture.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        GroundPicture.page().should eq 1
      end
      it '0以下の場合は1になる' do
        GroundPicture.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        GroundPicture.page_size('7').should eq 7
      end
      it 'nilの場合はGroundPicture.default_page_sizeになる' do
        GroundPicture.page_size().should eq GroundPicture.default_page_size
      end
      it '0以下の場合はGroundPicture.default_page_sizeになる' do
        GroundPicture.page_size('0').should eq GroundPicture.default_page_size
      end
      it 'GroundPicture.max_page_sizeを超えた場合はGroundPicture.max_page_sizeになる' do
        GroundPicture.page_size('1000').should eq GroundPicture.max_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        GroundPicture.stub(:list_opt).with(any_args).and_return({:include => :panel})
        GroundPicture.should_receive(:list_opt).with(any_args).exactly(1)
        r = GroundPicture.list
      end
    end
    it 'リストを返す' do
      pl = GroundPicture.list
      pl.should eq [@gp]
    end
    it '時系列で並んでいる' do
      #公開コマなら(他人のコマであっても)含んでいる
      opl = FactoryGirl.create :panel, :author_id => @other_author.id, :publish => 1
      npl = FactoryGirl.create :ground_picture, :panel_id => opl.id, :picture_id => @p.id, :updated_at => Time.now + 100
      pl = GroundPicture.list
      pl.should eq [npl, @gp]
    end
    it '非公開のコマの景色は含まない' do
      hpl = FactoryGirl.create :panel, :author_id => @author.id, :publish => 0
      npl = FactoryGirl.create :ground_picture, :panel_id => hpl.id, :picture_id => @p.id, :updated_at => Time.now + 100
      pl = GroundPicture.list
      pl.should eq [@gp]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @gp2 = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :z => 1, :picture_id => @p.id, :updated_at => Time.now + 100
        @gp3 = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :z => 2, :picture_id => @p.id, :updated_at => Time.now + 200
        @gp4 = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :z => 3, :picture_id => @p.id, :updated_at => Time.now + 300
        @gp5 = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :z => 4, :picture_id => @p.id, :updated_at => Time.now + 400
        GroundPicture.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        pl = GroundPicture.list
        pl.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        pl = GroundPicture.list(1)
        pl.should eq [@gp5, @gp4]
      end
      it 'page=2なら中間2件を返す' do
        pl = GroundPicture.list(2)
        pl.should eq [@gp3, @gp2]
      end
      it 'page=3なら先頭1件を返す' do
        pl = GroundPicture.list(3)
        pl.should eq [@gp]
      end
    end
    context 'DBに5件あって1ページの件数を0件に変えたとして' do
      before do
        @gp2 = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :z => 1, :picture_id => @p.id, :updated_at => Time.now + 100
        @gp3 = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :z => 2, :picture_id => @p.id, :updated_at => Time.now + 200
        @gp4 = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :z => 3, :picture_id => @p.id, :updated_at => Time.now + 300
        @gp5 = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :z => 4, :picture_id => @p.id, :updated_at => Time.now + 400
        GroundPicture.stub(:default_page_size).and_return(2)
      end
      it '通常は全件(5件)を返す' do
        r = GroundPicture.list 5, 0
        r.should have(5).items 
      end
    end
  end
  describe '一覧取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = GroundPicture.list_opt
      r.has_key?(:include).should be_true
    end
    it '2つの項目を含んでいる' do
      r = GroundPicture.list_opt[:include]
      r.should have(2).items
    end
    it 'コマを含んでいる' do
      r = GroundPicture.list_opt[:include]
      r.has_key?(:panel).should be_true
    end
      it 'コマは作家を含んでいる' do
        r = GroundPicture.list_opt[:include]
        r[:panel].has_key?(:author).should be_true
      end
    it '実素材を含んでいる' do
      r = GroundPicture.list_opt[:include]
      r.has_key?(:picture).should be_true
    end
      it '実素材は絵師を含んでいる' do
        r = GroundPicture.list_opt[:include]
        r[:picture].has_key?(:artist).should be_true
      end
      it '実素材はライセンスを含んでいる' do
        r = GroundPicture.list_opt[:include]
        r[:picture].has_key?(:license).should be_true
      end
  end
  describe 'json一覧出力オプションに於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id, :picture_id => @p.id
      @sbt = FactoryGirl.create :speech_balloon_template
      @comic = FactoryGirl.create :comic, :author_id => @author.id, :visible => 1
      @panel = FactoryGirl.create :panel, :author_id => @author.id, :publish => 1
      @story = FactoryGirl.create :story, :author_id => @author.id, :comic_id => @comic.id, :panel_id => @panel.id
      @gp = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :picture_id => @p.id
    end
    it 'コマを含んでいる' do
      r = GroundPicture.list.to_json GroundPicture.list_json_opt
      j = JSON.parse r
      i = j.first
      i.has_key?('panel').should be_true
    end
      it 'コマは作家を含んでいる' do
        r = GroundPicture.list.to_json GroundPicture.list_json_opt
        j = JSON.parse r
        i = j.first
        s = i['panel']
        s.has_key?('author').should be_true
      end
    it '実素材を含んでいる' do
      r = GroundPicture.list.to_json GroundPicture.list_json_opt
      j = JSON.parse r
      i = j.first
      i.has_key?('picture').should be_true
    end
      it '実素材は絵師を含んでいる' do
        r = GroundPicture.list.to_json GroundPicture.list_json_opt
        j = JSON.parse r
        i = j.first
        s = i['picture']
        s.has_key?('artist').should be_true
      end
      it '実素材はライセンスを含んでいる' do
        r = GroundPicture.list.to_json GroundPicture.list_json_opt
        j = JSON.parse r
        i = j.first
        s = i['picture']
        s.has_key?('license').should be_true
      end
  end
  
  describe '自分のコマで使った景色画像一覧取得に於いて' do
    before do
      @gp = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :picture_id => @p.id
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        GroundPicture.stub(:list_opt).with(any_args).and_return({:include => :panel})
        GroundPicture.should_receive(:list_opt).with(any_args).exactly(1)
        r = GroundPicture.mylist @author
      end
    end
    it 'リストを返す' do
      pl = GroundPicture.mylist @author
      pl.should eq [@gp]
    end
    it '時系列で並んでいる' do
      npl = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :picture_id => @p.id, :z => 2, :updated_at => Time.now + 100
      pl = GroundPicture.mylist @author
      pl.should eq [npl, @gp]
    end
    it '他人のコマの景色は公開でも含まない' do
      hpl = FactoryGirl.create :panel, :author_id => @other_author.id, :publish => 1
      npl = FactoryGirl.create :ground_picture, :panel_id => hpl.id, :picture_id => @p.id
      pl = GroundPicture.mylist @author
      pl.should eq [@gp]
    end
    it '自分のコマの景色は非公開でも含んでいる' do
      hpl = FactoryGirl.create :panel, :author_id => @author.id, :publish => 0
      npl = FactoryGirl.create :ground_picture, :panel_id => hpl.id, :picture_id => @p.id, :z => 2, :updated_at => Time.now + 100
      pl = GroundPicture.mylist @author
      pl.should eq [npl, @gp]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @gp2 = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :z => 1, :picture_id => @p.id, :updated_at => Time.now + 100
        @gp3 = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :z => 2, :picture_id => @p.id, :updated_at => Time.now + 200
        @gp4 = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :z => 3, :picture_id => @p.id, :updated_at => Time.now + 300
        @gp5 = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :z => 4, :picture_id => @p.id, :updated_at => Time.now + 400
      end
      it '通常は2件を返す' do
        c = GroundPicture.mylist @author, 1, 2
        c.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        c = GroundPicture.mylist(@author, 1, 2)
        c.should eq [@gp5, @gp4]
      end
      it 'page=2なら中間2件を返す' do
        c = GroundPicture.mylist(@author, 2, 2)
        c.should eq [@gp3, @gp2]
      end
      it 'page=3なら先頭1件を返す' do
        c = GroundPicture.mylist(@author, 3, 2)
        c.should eq [@gp]
      end
    end
    context 'DBに5件あって1ページの件数を0件に変えたとして' do
      before do
        @gp2 = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :z => 1, :picture_id => @p.id, :updated_at => Time.now + 100
        @gp3 = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :z => 2, :picture_id => @p.id, :updated_at => Time.now + 200
        @gp4 = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :z => 3, :picture_id => @p.id, :updated_at => Time.now + 300
        @gp5 = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :z => 4, :picture_id => @p.id, :updated_at => Time.now + 400
        Author.stub(:default_ground_picture_page_size).and_return(2)
      end
      it '通常は全件(5件)を返す' do
        r = GroundPicture.mylist @author, 5, 0
        r.should have(5).items 
      end
    end
  end
  
  
end
